# -*- coding: utf-8 -*-
'''
This module provides utilities for working with :term:`URIs <URI>`.
'''

import rfc3987

def is_uri(uri):
    '''
    Check if a string is a valid URI according to rfc3987

    :param string uri:
    :rtype: boolean
    '''
    if uri is None:
        return False
    return rfc3987.match(str(uri), rule='URI')

def get_id(uri):
    '''
    Splits the id from a standard Onroerend Erfgoed :term:`URI`

    :param string uri: A standard Onroerend Erfgoed :term:`URI`
    :rtype: string
    '''
    return uri.split('/')[-1]



# -*- coding: utf-8 -*-
'''
This module provides easy access to resources
'''

import logging
log = logging.getLogger(__name__)

import requests

from monnik.core import get_resource

from monnik.uri import is_uri

HOSTS = {
    'uri': 'https://id.erfgoed.net',
    'inventaris': 'https://inventaris.onroerenderfgoed.be'
}

URI_TEMPLATES = {
    'erfgoedobject': HOSTS["uri"] + '/erfgoedobjecten/{identifier}',
    'aanduidingsobject': HOSTS["uri"] + '/aanduidingsobjecten/{identifier}',
    'waarneming': HOSTS["uri"] + '/waarnemingen/{identifier}',
    'thema': HOSTS["uri"] + '/persoon/{identifier}',
    'gebeurtenis': HOSTS["uri"] + '/persoon/{identifier}',
    'persoon': HOSTS["uri"] + '/persoon/{identifier}',
}

URL_TEMPLATES = {
    'erfgoedobject': HOSTS['inventaris'] + '/erfgoedobjecten/{identifier}',
    'aanduidingsobject': HOSTS['inventaris'] + '/aanduidingsobjecten/{identifier}',
    'waarneming': HOSTS['inventaris'] + '/waarnemingen/{identifier}',
    'thema': HOSTS['inventaris'] + '/thema/{identifier}',
    'gebeurtenis': HOSTS['inventaris'] + '/gebeurtenis{identifier}',
    'persoon': HOSTS['inventaris'] + '/persoon/{identifier}',
}

class Monnik:

    session = None

    def __init__(self, session = None):
        if not session:
            session = requests.Session()
        self.session = session

    def _get_resource(self, identifier, resourcetype):
        if not is_uri(identifier):
            identifier = URI_TEMPLATES[resourcetype].format(identifier=identifier)
        return get_resource(identifier, session = self.session)

    def get_erfgoedobject(self, identifier):
        return self._get_resource(identifier, 'erfgoedobject')

    def get_aanduidingsobject(self, identifier):
        return self._get_resource(identifier, 'aanduidingsobject')

    def get_waarneming(self, identifier):
        return self._get_resource(identifier, 'waarneming')

    def get_thema(self, identifier):
        return self._get_resource(identifier, 'thema')

    def get_gebeurtenis(self, identifier):
        return self._get_resource(identifier, 'gebeurtenis')

    def get_persoon(self, identifier):
        return self._get_resource(identifier, 'persoon')



#!/usr/bin/env python

import os
import sys

import monnik

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

packages = [
    'monnik',
]

requires = [
    'rfc3987',
    'requests',
]

setup(
    name='Monnik',
    version='0.1.0-alpha',
    description='Library for easy querying of Flanders Heritage REST services.',
    long_description=open('README.rst').read(),
    long_description_content_type='text/x-rst',
    author='Koen Van Daele',
    author_email='koen.vandaele@vlaanderen.be',
    url='http://github.com/onroerenderfgoed/monnik',
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'monnik': 'monnik'},
    include_package_data=True,
    install_requires=requires,
    license='MIT',
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ]
)



# -*- coding: utf-8 -*-
'''
This module provides utilities for doing requests.
'''

import requests

import logging
log = logging.getLogger(__name__)

from .uri import is_uri

RETRY_ATTEMPTS = 5
TIMEOUT = 10

def get_resource(url, SSO=None, session=None):
    '''
    Get a single resource. Any object that is represented by a single url,
    without extra parameters.

    :param str url: The url of the resource
    :param str SSO: An OpenAm SSO key
    :param requests.Session session: A session that can be reused.
    :return: Returns a dictionary with data or False if the resource could not
    be reached
    :rtype: dict
    :raises Exception: if the url could not be reached
    :raises ValueError: if the url parameter is not a valid url
    '''

    if not is_uri(url):
        raise ValueError(
            f'{url} is not a valid url'
        )

    if not session:
        session = requests.Session()

    headers = {
        'Accept': 'application/json'
    }
    if SSO:
        headers['OpenAmSSOID'] = SSO

    for i in range(RETRY_ATTEMPTS):  # Try up to RETRY_ATTEMPTS times to get the URI
        # Retry scenarios are: connection errors, timeouts, 5xx status codes
        try:
            response = session.get(url, headers=headers, timeout=TIMEOUT)
            if response.ok:
                return response.json()
            if 400 <= response.status_code < 500:
                return False
            else:
                log.error(
                    f"Problem fetching {url}: "
                    f"{response.status_code} - {response.text}"
                )
        except RequestException as e:
            log.error(
                f"Problem fetching {url}: {e}"
            )
    else:  # When the loop exited without a break, all retries used.
        raise Exception(f'Tried to reach {url} {RETRY_ATTEMPT} times, but failed.')

def get_collection(url, parameters, SSO = None, session=None):
    '''
    Fetch all data from a url until there are no more `next` urls in the Link
    header.

    :param str url: The url of the collecion
    :param dict parameters: A dict of query string parameters
    :param str SSO: An OpenAm SSO key
    :return: Returns a list of resources that form the collection
    :rtype: lst
    '''

    if not session:
        session = requests.Session()

    data = []

    headers = {
        'Accept': 'application/json'
    }
    if SSO:
        headers['OpenAmSSOID'] = SSO

    def _do_query(url, params, headers):
        for i in range(RETRY_ATTEMPTS):  # Try up to RETRY_ATTEMPTS times to get the URI
            # Retry scenarios are: connection errors, timeouts, 5xx status codes
            try:
                response = session.get(url, params=params, headers=headers, timeout=TIMEOUT)
                if response.ok:
                    return response
                if 400 <= response.status_code < 500:
                    log.error(
                        f"Client error, please check your query: "
                        f"{response.status_code} - {response.text}"
                    )
                    # No point in retrying
                    raise Exception(f'Could not exceute query: {response.status_code} - {response.text}')
                else:
                    log.error(
                        f"Problem fetching {url}: "
                        f"{response.status_code} - {response.text}"
                    )
            except RequestException as e:
                log.error(
                    f"Problem fetching {url}: {e}"
                )
        else:  # When the loop exited without a break, all retries used.
            raise Exception(f'Tried to reach {url} {RETRY_ATTEMPT} times, but failed.')

    res = _do_query(url, parameters, headers)

    total = None
    if res.status_code == requests.codes.ok:
        if 'Content-Range' in res.headers:
            total = res.headers['Content-Range'].split('/')[-1]
        else:
            total = 0
        log.debug(f'Planning to get {total} results', total)

    if not total:
        log.info('No results found')
        return []
    else:
        data.extend(res.json())

    while 'next' in res.links:
        res = _do_query(res.links['next']['url'], params={}, headers=headers)
        data.extend(res.json())

    return data
